package ArchiveContent;

use strict;
use warnings;

use Logging;

use vars qw|@ISA|;

sub new {
  my $self = {};
  bless( $self, shift );
  $self->_init(@_);
  return $self;
}

sub _init {
  my ( $self, $storage, $packer ) = @_;
  $self->{storage} = $storage;
  $self->{packer} = $packer;
  Logging::info("Archive content transport has been initialized");
}

### Public routines ###
# addAdminContent,addResellerContent,addClientContent,addDomainContent,addDomainMailListContent,addPhostingContent,\
# addSubDomainContent,addSubDomainPhostingContent,addMailnameContent,addDbContent,addSubdomainDbContent

sub addAdminContent{
  my $self = shift;
  my $cid_type = shift;
  my $id = shift;
  my $file = shift;
  return $self->_addTar( $cid_type, $self->{packer}->getAdminBackupPath( $file ) , @_);
}

sub addResellerContent{
  my $self = shift;
  my $cid_type = shift;
  my $id = shift;
  my $file = shift;
  return $self->_addTar( $cid_type, $self->{packer}->getResellersBackupPath($id, $file ), @_);
}

sub addClientContent{
  my $self = shift;
  my $cid_type = shift;
  my $id = shift;
  my $file = shift;
  return $self->addResellerContent( $cid_type, $id, $file, @_ ) if defined ($self->{packer}->{resellersNodes}) and exists $self->{packer}->{resellersNodes}->{$id};
  return $self->_addTar( $cid_type, $self->{packer}->getClientsBackupPath($id, $file ), @_);
}

my @exclude_vhost_rootdirs = ('conf', 
                              'pd',
                              'statistics',
                              'statistics/webstat', 
                              'statistics/webstat-ssl', 
                              'statistics/ftpstat', 
                              'statistics/logs', 
                              'statistics/anon_ftpstat' 
                              );

sub addDomainContent{
  my $self = shift;
  my $cid_type = shift;
  my $id = shift;
  my $file = shift;
  if ($cid_type eq 'vhost') {
    return $self->_addTar( 'vhost', $self->{packer}->getDomainsBackupPath($id, $file ), @_, "exclude" => \@exclude_vhost_rootdirs);
  }
  else {
    return $self->_addTar( $cid_type, $self->{packer}->getDomainsBackupPath($id, $file ), @_);
  }
}

sub addDomainMailListContent{
  my $self = shift;
  my $cid_type = shift;
  my $id = shift;
  my $file = shift;
  my $referer = $self->_getDomainMlTarReferer( $id, @_ );
  if( defined $referer ) {
    return $self->_addRefererCid( $referer , $cid_type);
  }
  else{
    return $self->_addTar( $cid_type, $self->{packer}->getDomainsBackupPath($id, $file ), @_);
  }
}

sub addPhostingContent{
  my $self = shift;
  my $cid_type = shift;
  my $id = shift;
  my $referralId = shift;
  my $file = shift;
  my $referer = $self->_getDomainPhTarReferer( $self->{packer}->getPhostingParentId( $referralId ), @_ );
  $referer = $self->_getDomainStatTarReferer( $self->{packer}->getPhostingParentId( $referralId ), @_ ) unless defined $referer;
  if( defined $referer ) {
    return $self->_addRefererCid( $referer , $cid_type);
  }
  else{
    return $self->_addTar( $cid_type, $self->{packer}->getPhostingBackupPath($id, $file ), @_);
  }
}

sub addSubDomainContent{
  my $self = shift;
  my $cid_type = shift;
  my $id = shift;
  my $file = shift;
  return $self->_addTar( $cid_type, $self->{packer}->getSubdomainsBackupPath($id, $file ), @_);
}

sub addSubDomainPhostingContent{
  my $self = shift;
  my $cid_type = shift;
  my $id = shift;
  my $referralId = shift;
  my $file = shift;
  my $referer;
  if( $referralId ) {
    $referer = $self->_getDomainPhTarReferer( $referralId, @_ );
    unless( defined $referer) {
      $referer = $self->_getDomainPhTarReferer( $self->{packer}->getSubdomainParentId( $id ), @_ );
    }
  }
  if( defined $referer ) {
    return $self->_addRefererCid( $referer , $cid_type );
  }
  else {
    return $self->_addTar( $cid_type, $self->{packer}->getSubdomainsBackupPath($id, $file ), @_);
  }
}

sub addMailnameContent{
  my $self = shift;
  my $cid_type = shift;
  my $id = shift;
  my $file = shift;
  my $domainId = $self->{packer}->getMailNameParentId( $id );
  my $referrer = $self->_getDomainMailTarReferer( $self->{packer}->getMailNameParentId( $id ), @_ );
  if( defined $referrer ){
    return $self->_addRefererCid( $referrer, $cid_type );
  }
  else {
    return $self->_addTar( $cid_type, $self->{packer}->getMailnamesBackupPath($id, $file ), @_);
  }
}

sub addDbContent {
  my $self = shift;
  my $cid_type = shift;
  my $id = shift;
  return $self->_addDb( $cid_type, $self->{packer}->getDatabasesBackupPath( $id, '' ), @_);
}

sub addSubdomainDbContent {
  my $self = shift;
  my $cid_type = shift;
  my $subdomainId = shift;
  my $id = shift;
  return $self->_addDb( $cid_type, $self->{packer}->getSubdomainsBackupPath( $subdomainId, "db$id" ), @_);
}

sub getContentTransportDescription {
  return undef;
}

### Private routines ###

sub _addTar {
  my $self = shift;
  my $cid_type = shift;
  my $dstFile = shift;
  return undef if exists $self->{packer}->{skip_content};
  my $id = $self->{storage}->addTar( $dstFile, @_);
  return undef unless defined $id;
  return $self->_makeCidNode( $id, $cid_type );
}

sub _getTarReferer{
  my( $self, $referrer, %options ) = @_;
  if( $referrer && exists $options{'directory'} ) {
    if( index( $options{'directory'}, $referrer->[2] ) == 0 ){
      my @res;
      my $ref = substr( $options{'directory'}, length($referrer->[2]) );
      while( index( $ref, '/' )==0 ) {
        $ref = substr( $ref, 1 );
      }
      if( $referrer->[0] eq 'vhost' and grep ( index($ref,$_ ) == 0, @exclude_vhost_rootdirs)) {
        return undef;
      }
      push @res, $referrer->[1];
      push @res, $ref;
      push @res, $options{'directory'};
      return \@res;
    }
  }
  return undef;
}

sub _getDomainMlTarReferer{
  my $self = shift;
  my $domainId = shift;
  return undef unless exists $self->{packer}->{domainMailListContent}->{$domainId};
  return $self->_getTarReferer( \@{$self->{packer}->{domainMailListContent}->{$domainId}}, @_ );
}

sub _getDomainPhTarReferer{
  my $self = shift;
  my $domainId = shift;
  return undef unless exists $self->{packer}->{domainVhost}->{$domainId};
  return $self->_getTarReferer( \@{$self->{packer}->{domainVhost}->{$domainId}}, @_ );
}

sub _getDomainStatTarReferer{
  my $self = shift;
  my $domainId = shift;
  return undef unless exists $self->{packer}->{domainStatistics}->{$domainId};
  return $self->_getTarReferer( \@{$self->{packer}->{domainStatistics}->{$domainId}}, @_ );
}

sub _getDomainMailTarReferer{
  my $self = shift;
  my $domainId = shift;
  return undef unless exists $self->{packer}->{domainMailContent}->{$domainId};
  return $self->_getTarReferer( \@{$self->{packer}->{domainMailContent}->{$domainId}}, @_ );
}

sub _addDb {
  my $self = shift;
  my $cid_type = shift;
  my $dstFile = shift;
  return undef if exists $self->{packer}->{skip_content};
  my $id = $self->{storage}->addDb( $dstFile, @_);
  return undef unless defined $id;
  return $self->_makeCidNode( $id, $cid_type);
}

sub _makeCidNode{
  my ($self, $id, $cid_type ) = @_;
  my $cid = XmlNode->new( 'cid' );
  $cid->setAttribute( 'type', $cid_type );
  $cid->setAttribute( 'unpacksize', $self->{storage}->getFilesUnpackSizeFromId($id) );
  $cid->setAttribute( 'path', $self->{storage}->getFilePathFromId( $id ) );
  my $files = $self->{storage}->getFilesFromId( $id );
  foreach my $filedata( @{$files} ){
    $cid->addChild( XmlNode->new( 'content-file', 'content' => $filedata->[0], 'attributes' => {'size' => $filedata->[1] } ) );
  }
  return $cid;
}

sub _addRefererCid{
  my ($self, $referer, $cid_type ) = @_;
  return undef unless defined $referer;
  Logging::info( "Add referrer cid for type '$cid_type', referer '$referer->[1]', from path '$referer->[2]'" );
  if( -e $referer->[2] ){
    my $cid = XmlNode->new( 'cid' );
    $cid->setAttribute( 'type', $cid_type );
    $cid->setAttribute( 'unpacksize', $referer->[0]->getAttribute( 'unpacksize' ) );
    $cid->setAttribute( 'path', $referer->[0]->getAttribute( 'path' ) );
    $cid->setAttribute( 'referrer', 'true' );
    $cid->setAttribute( 'offset', $referer->[1] );
    my @files = $referer->[0]->getChildren( 'content-file' );
    foreach my $file(@files) {
      $cid->addChild( XmlNode->new( 'content-file', 'content' => $file->getText(), 'attributes' => {'size' => $file->getAttribute('size') } ) ) if $file;
    }
    return $cid;
  }
  else{
    Logging::info( "The path  '$referer->[2]' is not exists. Skip it." );
  }
}


1;